from collections.abc import Sequence

from bs4 import BeautifulSoup
from django.core import mail
from django.core.mail.backends.smtp import EmailBackend

from .models import EmailConfiguration


class DynamicSMPTEmailBackend(EmailBackend):
    def __init__(
        self,
        host: str | None = None,
        port: str | None = None,
        username: str | None = None,
        password: str | None = None,
        use_tls: str | None = None,
        fail_silently: bool = False,
        use_ssl: str | None = None,
        timeout: str | None = None,
        ssl_keyfile: str | None = None,
        ssl_certfile: str | None = None,
        **kwargs: ...,
    ):
        conf = EmailConfiguration.get_solo()

        super().__init__(
            host=host or conf.host,
            port=port or conf.port,
            username=username or conf.username,
            password=password or conf.password,
            use_tls=use_tls or conf.use_tls,
            fail_silently=fail_silently,
            use_ssl=use_ssl or conf.use_ssl,
            timeout=timeout or conf.timeout,
            ssl_keyfile=ssl_keyfile,
            ssl_certfile=ssl_certfile,
            **kwargs,
        )


def html2text(html: str):
    return BeautifulSoup(html, "lxml").get_text()


def send_mail(
    subject: str,
    message: str,
    recipients: Sequence[str] | str,
):
    mailconf = EmailConfiguration.get_solo()
    recipient_list = (
        recipients.split(",") if isinstance(recipients, str) else recipients
    )

    return mail.send_mail(
        subject,
        html2text(message),
        recipient_list=recipient_list,
        html_message=message,
        fail_silently=False,
        auth_user=mailconf.username,
        auth_password=mailconf.password,
        from_email=mailconf.from_email,
    )



from pathlib import Path

from django import setup
from django.conf import settings
from django.core.management import call_command

BASE_DIR = Path(__file__).resolve().parent


def boot_django():
    settings.configure(
        BASE_DIR=BASE_DIR,
        DEBUG=True,
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
                "NAME": BASE_DIR / "db.sqlite3",
            }
        },
        INSTALLED_APPS=(
            "django.contrib.auth",
            "django.contrib.contenttypes",
            "dynamic_smtp",
        ),
    )

    setup()


boot_django()
ret = call_command("makemigrations", "dynamic_smtp")
print(ret)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DynamicSmtpConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "dynamic_smtp"
    verbose_name = _("Email configuration")



from smtplib import SMTPException

from django.conf import settings
from django.contrib import admin, messages
from django.core import mail
from django.core.mail.backends.console import EmailBackend as ConsoleEmailBackend
from django.http import HttpRequest
from django.utils.translation import gettext_lazy as _
from django_object_actions import DjangoObjectActions, action
from solo.admin import SingletonModelAdmin

from .models import AbstractEmailConfiguration, EmailConfiguration


class EmailConfigurationAdmin(DjangoObjectActions, SingletonModelAdmin):
    change_actions = ["test_email"]

    @action(
        label=_("Test email"),
        description=_(
            "Send a test email from the account configured below to the "
            "email registered as sender. Save before using."
        ),
    )
    def test_email(self, request: HttpRequest, obj: AbstractEmailConfiguration):
        host = request.get_host()
        try:
            with mail.get_connection() as connection:
                if type(connection) is ConsoleEmailBackend:
                    raise AssertionError(
                        _("Using debug backend. Try using DEBUG=False in settings")
                    )
                __ = mail.send_mail(
                    message=_(
                        "This email has been sent to verify if the "
                        "mailing system is well configured in the "
                        "%(host)s's administration. It seems so!"
                    )
                    % {"host": host},
                    from_email=obj.from_email,
                    subject=_("[%(host)s] Email test") % {"host": host},
                    recipient_list=[obj.from_email],
                    fail_silently=False,
                    connection=connection,
                )
        except (SMTPException, AssertionError) as e:
            messages.error(
                request,
                _("Email not well configured: %s") % str(e),
            )
            return

        messages.success(
            request,
            _("Email is well configured! :)"),
        )


if getattr(settings, "DYNAMIC_SMPT_REGISTER_ADMIN", True):
    admin.site.register(EmailConfiguration, EmailConfigurationAdmin)
